@@ -1,8 +1,8 @@
-// $Id: recorder.hpp 3757 2015-11-30 12:03:07Z bradbell $
-# ifndef CPPAD_RECORDER_HPP
-# define CPPAD_RECORDER_HPP
+// $Id: recorder.hpp 3845 2016-11-19 01:50:47Z bradbell $
+# ifndef CPPAD_LOCAL_RECORDER_HPP
+# define CPPAD_LOCAL_RECORDER_HPP
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-15 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the
@@ -11,10 +11,10 @@ the terms of the
 A copy of this license is included in the COPYING file of this distribution.
 Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
-# include <cppad/local/hash_code.hpp>
+# include <cppad/core/hash_code.hpp>
 # include <cppad/local/pod_vector.hpp>
 
-namespace CppAD { // BEGIN_CPPAD_NAMESPACE
+namespace CppAD { namespace local { // BEGIN_CPPAD_LOCAL_NAMESPACE
 /*!
 \file recorder.hpp
 File used to define the recorder class.
@@ -583,5 +583,5 @@ inline size_t recorder<Base>::PutTxt(const char *text)
 // -------------------------------------------------------------------------
 
 
-} // END_CPPAD_NAMESPACE
+} } // END_CPPAD_LOCAL_NAMESPACE
 # endif
