@@ -1,9 +1,9 @@
-// $Id: rev_hes_sweep.hpp 3757 2015-11-30 12:03:07Z bradbell $
-# ifndef CPPAD_REV_HES_SWEEP_HPP
-# define CPPAD_REV_HES_SWEEP_HPP
+// $Id: rev_hes_sweep.hpp 3853 2016-12-14 14:40:11Z bradbell $
+# ifndef CPPAD_LOCAL_REV_HES_SWEEP_HPP
+# define CPPAD_LOCAL_REV_HES_SWEEP_HPP
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-15 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the
@@ -13,7 +13,7 @@ A copy of this license is included in the COPYING file of this distribution.
 Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
 
-namespace CppAD { // BEGIN_CPPAD_NAMESPACE
+namespace CppAD { namespace local { // BEGIN_CPPAD_LOCAL_NAMESPACE
 /*!
 \file rev_hes_sweep.hpp
 Compute Reverse mode Hessian sparsity patterns.
@@ -40,7 +40,7 @@ using AD< \a Base > and computations by this routine are done using type
 
 \tparam Vector_set
 is the type used for vectors of sets. It can be either
-\c sparse_pack, \c sparse_set, or \c sparse_list.
+sparse_pack or sparse_list.
 
 \param n
 is the number of independent variables on the tape.
@@ -61,7 +61,7 @@ where \f$ n \f$ is the number of independent variables
 and \f$ m \f$ is the number of dependent variables.
 The object \a play is effectly constant.
 It is not declared const because while playing back the tape
-the object \a play holds information about the currentl location
+the object \a play holds information about the current location
 with in the tape and this changes during playback.
 
 \param for_jac_sparse
@@ -102,8 +102,8 @@ template <class Base, class Vector_set>
 void RevHesSweep(
 	size_t                n,
 	size_t                numvar,
-	player<Base>         *play,
-	Vector_set&           for_jac_sparse, // should be const
+	local::player<Base>*         play,
+	const Vector_set&     for_jac_sparse,
 	bool*                 RevJac,
 	Vector_set&           rev_hes_sparse
 )
@@ -120,7 +120,7 @@ void RevHesSweep(
 	size_t             i, j, k;
 
 	// check numvar argument
-	CPPAD_ASSERT_UNKNOWN( play->num_var_rec()     == numvar );
+	CPPAD_ASSERT_UNKNOWN( play->num_var_rec()    == numvar );
 	CPPAD_ASSERT_UNKNOWN( for_jac_sparse.n_set() == numvar );
 	CPPAD_ASSERT_UNKNOWN( rev_hes_sparse.n_set() == numvar );
 	CPPAD_ASSERT_UNKNOWN( numvar > 0 );
@@ -165,7 +165,9 @@ void RevHesSweep(
 	}
 
 	// work space used by UserOp.
+	vector<Base>       user_x;   // parameters in x as integers;
 	vector<size_t>     user_ix;  // variable indices for argument vector x
+	//
 	typedef std::set<size_t> size_set;
 	size_set::iterator set_itr;  // iterator for a standard set
 	size_set::iterator set_end;  // end of iterator sequence
@@ -185,24 +187,23 @@ void RevHesSweep(
 	vector<bool>       user_s;   // reverse Jacobian sparsity for y
 	vector<bool>       user_t;   // reverse Jacobian sparsity for x
 	const size_t user_q = limit; // maximum element plus one
-	size_t user_index = 0;       // indentifier for this atomic operation
-	size_t user_id    = 0;       // user identifier for this call to operator
-	size_t user_i     = 0;       // index in result vector
-	size_t user_j     = 0;       // index in argument vector
-	size_t user_m     = 0;       // size of result vector
-	size_t user_n     = 0;       // size of arugment vector
 	//
 	atomic_base<Base>* user_atom = CPPAD_NULL; // user's atomic op calculator
 	bool               user_pack = false;      // sparsity pattern type is pack
 	bool               user_bool = false;      // sparsity pattern type is bool
 	bool               user_set  = false;      // sparsity pattern type is set
-# ifndef NDEBUG
 	bool               user_ok   = false;      // atomic op return value
-# endif
-	// next expected operator in a UserOp sequence
-	enum { user_start, user_arg, user_ret, user_end } user_state = user_end;
-
-
+	//
+	// information defined by forward_user
+	size_t user_old=0, user_m=0, user_n=0, user_i=0, user_j=0;
+	enum_user_state user_state = end_user; // proper initialization
+	//
+	// pointer to the beginning of the parameter vector
+	// (used by atomic functions
+	const Base* parameter = CPPAD_NULL;
+	if( num_par > 0 )
+		parameter = play->GetPar();
+	//
 	// Initialize
 	play->reverse_start(op, arg, i_op, i_var);
 	CPPAD_ASSERT_UNKNOWN( op == EndOp );
@@ -213,7 +214,8 @@ void RevHesSweep(
 # endif
 	bool more_operators = true;
 	while(more_operators)
-	{
+	{	bool flag; // temporary for use in switch cases
+		//
 		// next op
 		play->reverse_next(op, arg, i_op, i_var);
 # ifndef NDEBUG
@@ -392,7 +394,7 @@ void RevHesSweep(
 
 			case ErfOp:
 			// arg[1] is always the parameter 0
-			// arg[0] is always the parameter 2 / sqrt(pi)
+			// arg[2] is always the parameter 2 / sqrt(pi)
 			CPPAD_ASSERT_NARG_NRES(op, 3, 5);
 			reverse_sparse_hessian_nonlinear_unary_op(
 			i_var, arg[0], RevJac, for_jac_sparse, rev_hes_sparse
@@ -526,7 +528,7 @@ void RevHesSweep(
 
 			case PowvvOp:
 			CPPAD_ASSERT_NARG_NRES(op, 2, 3)
-                        reverse_sparse_hessian_pow_op(
+			reverse_sparse_hessian_pow_op(
 			i_var, arg, RevJac, for_jac_sparse, rev_hes_sparse
 			);
 			break;
@@ -653,29 +655,19 @@ void RevHesSweep(
 
 			case UserOp:
 			// start or end an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( NumRes( UserOp ) == 0 );
-			CPPAD_ASSERT_UNKNOWN( NumArg( UserOp ) == 4 );
-			if( user_state == user_end )
-			{	user_index = arg[0];
-				user_id    = arg[1];
-				user_n     = arg[2];
-				user_m     = arg[3];
-				user_atom  = atomic_base<Base>::class_object(user_index);
-# ifndef NDEBUG
-				if( user_atom == CPPAD_NULL )
-				{	std::string msg =
-						atomic_base<Base>::class_name(user_index)
-						+ ": atomic_base function has been deleted";
-					CPPAD_ASSERT_KNOWN(false, msg.c_str() );
-				}
-# endif
-				user_pack  = user_atom->sparsity() ==
+			flag = user_state == end_user;
+			user_atom = play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
+			if( flag )
+			{	user_pack  = user_atom->sparsity() ==
 							atomic_base<Base>::pack_sparsity_enum;
 				user_bool  = user_atom->sparsity() ==
 							atomic_base<Base>::bool_sparsity_enum;
 				user_set   = user_atom->sparsity() ==
 							atomic_base<Base>::set_sparsity_enum;
 				CPPAD_ASSERT_UNKNOWN( user_pack || user_bool || user_set );
+				user_x.resize(user_n);
 				user_ix.resize(user_n);
 				user_vx.resize(user_n);
 				user_s.resize(user_m);
@@ -733,49 +725,37 @@ void RevHesSweep(
 						set_v[i].clear();
 					}
 				}
-				user_j     = user_n;
-				user_i     = user_m;
-				user_state = user_ret;
 			}
 			else
-			{	CPPAD_ASSERT_UNKNOWN( user_state == user_start );
-				CPPAD_ASSERT_UNKNOWN( user_index == size_t(arg[0]) );
-				CPPAD_ASSERT_UNKNOWN( user_id    == size_t(arg[1]) );
-				CPPAD_ASSERT_UNKNOWN( user_n     == size_t(arg[2]) );
-				CPPAD_ASSERT_UNKNOWN( user_m     == size_t(arg[3]) );
-				user_state = user_end;
-
-				// call users function for this operation
-				user_atom->set_id(user_id);
-# ifdef NDEBUG
+			{	// call users function for this operation
+				user_atom->set_old(user_old);
 				if( user_pack )
-					user_atom->rev_sparse_hes(user_vx,
-						user_s, user_t, user_q, pack_r, pack_u, pack_v
-				);
+				{	user_ok = user_atom->rev_sparse_hes(user_vx,
+						user_s, user_t, user_q, pack_r, pack_u, pack_v, user_x
+					);
+					if( ! user_ok ) user_ok = user_atom->rev_sparse_hes(
+						user_vx, user_s, user_t, user_q, pack_r, pack_u, pack_v
+					);
+				}
 				if( user_bool )
-					user_atom->rev_sparse_hes(user_vx,
-						user_s, user_t, user_q, bool_r, bool_u, bool_v
-				);
+				{	user_ok = user_atom->rev_sparse_hes(user_vx,
+						user_s, user_t, user_q, bool_r, bool_u, bool_v, user_x
+					);
+					if( ! user_ok ) user_ok = user_atom->rev_sparse_hes(
+						user_vx, user_s, user_t, user_q, bool_r, bool_u, bool_v
+					);
+				}
 				if( user_set )
-					user_atom->rev_sparse_hes(user_vx,
-						user_s, user_t, user_q, set_r, set_u, set_v
-				);
-# else
-				if( user_pack )
-					user_ok = user_atom->rev_sparse_hes(user_vx,
-						user_s, user_t, user_q, pack_r, pack_u, pack_v
-				);
-				if( user_bool )
-					user_ok = user_atom->rev_sparse_hes(user_vx,
-						user_s, user_t, user_q, bool_r, bool_u, bool_v
-				);
-				if( user_set )
-					user_ok = user_atom->rev_sparse_hes(user_vx,
-						user_s, user_t, user_q, set_r, set_u, set_v
-				);
+				{	user_ok = user_atom->rev_sparse_hes(user_vx,
+						user_s, user_t, user_q, set_r, set_u, set_v, user_x
+					);
+					if( ! user_ok ) user_ok = user_atom->rev_sparse_hes(
+						user_vx, user_s, user_t, user_q, set_r, set_u, set_v
+					);
+				}
 				if( ! user_ok )
 				{	std::string msg =
-						atomic_base<Base>::class_name(user_index)
+						user_atom->afun_name()
 						+ ": atomic_base.rev_sparse_hes: returned false\n";
 					if( user_pack )
 						msg += "sparsity = pack_sparsity_enum";
@@ -785,7 +765,6 @@ void RevHesSweep(
 						msg += "sparsity = set_sparsity_enum";
 					CPPAD_ASSERT_KNOWN(false, msg.c_str() );
 				}
-# endif
 				for(i = 0; i < user_n; i++) if( user_ix[i] > 0 )
 				{
 					size_t  i_x = user_ix[i];
@@ -811,80 +790,79 @@ void RevHesSweep(
 							rev_hes_sparse.add_element(i_x, *set_itr++);
 					}
 				}
-               }
+			}
 			break;
 
 			case UsrapOp:
 			// parameter argument in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_arg );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_j && user_j <= user_n );
-			CPPAD_ASSERT_UNKNOWN( NumArg(op) == 1 );
 			CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) < num_par );
-			--user_j;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			user_ix[user_j] = 0;
 			user_vx[user_j] = false;
-			if( user_j == 0 )
-				user_state = user_start;
+			//
+			// parameters as integers
+			user_x[user_j] = parameter[arg[0]];
+			//
 			break;
 
 			case UsravOp:
 			// variable argument in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_arg );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_j && user_j <= user_n );
-			CPPAD_ASSERT_UNKNOWN( NumArg(op) == 1 );
 			CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) <= i_var );
 			CPPAD_ASSERT_UNKNOWN( 0 < arg[0] );
-			--user_j;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			user_ix[user_j] = arg[0];
 			user_vx[user_j] = true;
-			for_jac_sparse.begin(arg[0]);
-			i = for_jac_sparse.next_element();
-			while( i < user_q )
-			{	if( user_pack )
-					pack_r[ user_j * user_q + i ] = true;
-				if( user_bool )
-					bool_r[ user_j * user_q + i ] = true;
-				if( user_set )
-					set_r[user_j].insert(i);
-				i = for_jac_sparse.next_element();
+			//
+			// variable as integers
+			user_x[user_j] = CppAD::numeric_limits<Base>::quiet_NaN();
+			//
+			{	typename Vector_set::const_iterator itr(for_jac_sparse, arg[0]);
+				i = *itr;
+				while( i < user_q )
+				{	if( user_pack )
+						pack_r[ user_j * user_q + i ] = true;
+					if( user_bool )
+						bool_r[ user_j * user_q + i ] = true;
+					if( user_set )
+						set_r[user_j].insert(i);
+					i = *(++itr);
+				}
 			}
-			if( user_j == 0 )
-				user_state = user_start;
 			break;
 
 			case UsrrpOp:
 			// parameter result in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_ret );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_i && user_i <= user_m );
-			CPPAD_ASSERT_UNKNOWN( NumArg(op) == 1 );
 			CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) < num_par );
-			--user_i;
-			if( user_i == 0 )
-				user_state = user_arg;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			break;
 
 			case UsrrvOp:
 			// variable result in an atomic operation sequence
-			CPPAD_ASSERT_UNKNOWN( user_state == user_ret );
-			CPPAD_ASSERT_UNKNOWN( 0 < user_i && user_i <= user_m );
-			--user_i;
+			play->reverse_user(op, user_state,
+				user_old, user_m, user_n, user_i, user_j
+			);
 			if( RevJac[i_var] )
 			{
 				user_s[user_i] = true;
 			}
-			rev_hes_sparse.begin(i_var);
-			j = rev_hes_sparse.next_element();
-			while( j < user_q )
-			{	if( user_pack )
-					pack_u[user_i * user_q + j] = true;
-				if( user_bool )
-					bool_u[user_i * user_q + j] = true;
-				if( user_set )
-					set_u[user_i].insert(j);
-				j = rev_hes_sparse.next_element();
+			{	typename Vector_set::const_iterator itr(rev_hes_sparse, i_var);
+				j = *itr;
+				while( j < user_q )
+				{	if( user_pack )
+						pack_u[user_i * user_q + j] = true;
+					if( user_bool )
+						bool_u[user_i * user_q + j] = true;
+					if( user_set )
+						set_u[user_i].insert(j);
+					j = *(++itr);
+				}
 			}
-			if( user_i == 0 )
-				user_state = user_arg;
 			break;
 			// -------------------------------------------------
 
@@ -921,17 +899,17 @@ void RevHesSweep(
 		{	zf_value[j] = false;
 			zh_value[j] = false;
 		}
-		for_jac_sparse.begin(i_var);;
-		j = for_jac_sparse.next_element();;
+		typename Vector_set::const_iterator itr_jac(for_jac_sparse, i_var);
+		j = *itr_jac;
 		while( j < limit )
 		{	zf_value[j] = true;
-			j = for_jac_sparse.next_element();
+			j = *(++itr_jac);
 		}
-		rev_hes_sparse.begin(i_var);;
-		j = rev_hes_sparse.next_element();;
+		typename Vector_set::const_iterator itr_hes(rev_hes_sparse, i_var);
+		j = *itr_hes;
 		while( j < limit )
 		{	zh_value[j] = true;
-			j = rev_hes_sparse.next_element();
+			j = *(++itr_hes);
 		}
 		printOp(
 			std::cout,
@@ -962,7 +940,7 @@ void RevHesSweep(
 
 	return;
 }
-} // END_CPPAD_NAMESPACE
+} } // END_CPPAD_LOCAL_NAMESPACE
 
 // preprocessor symbols that are local to this file
 # undef CPPAD_REV_HES_SWEEP_TRACE
