@@ -1,6 +1,6 @@
-/* $Id: asin.cpp 3675 2015-05-05 14:26:18Z bradbell $ */
+// $Id: asin.cpp 3788 2016-02-09 15:50:06Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-15 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the
@@ -20,7 +20,7 @@ $$
 $section The AD asin Function: Example and Test$$
 
 $code
-$verbatim%example/asin.cpp%0%// BEGIN C++%// END C++%1%$$
+$srcfile%example/asin.cpp%0%// BEGIN C++%// END C++%1%$$
 $$
 
 $end
