@@ -1,6 +1,6 @@
-// $Id: ode_gear.cpp 3757 2015-11-30 12:03:07Z bradbell $
+// $Id: ode_gear.cpp 3788 2016-02-09 15:50:06Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-15 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the
@@ -37,7 +37,7 @@ x_i '(t)   & = & (i+1) t^i = (i+1) x_{i-1} (t) & {\rm if \;} i > 0
 The example tests OdeGear using the relations above:
 
 $code
-$verbatim%example/ode_gear.cpp%0%// BEGIN C++%// END C++%1%$$
+$srcfile%example/ode_gear.cpp%0%// BEGIN C++%// END C++%1%$$
 $$
 
 $end
