@@ -1,6 +1,6 @@
-// $Id: simple_ad_bthread.cpp 3757 2015-11-30 12:03:07Z bradbell $
+// $Id: simple_ad_bthread.cpp 3788 2016-02-09 15:50:06Z bradbell $
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-15 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-16 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the
@@ -27,7 +27,7 @@ boost multi-threading environment.
 
 $head Source Code$$
 $code
-$verbatim%multi_thread/bthread/simple_ad_bthread.cpp%0%// BEGIN C++%// END C++%1%$$
+$srcfile%multi_thread/bthread/simple_ad_bthread.cpp%0%// BEGIN C++%// END C++%1%$$
 $$
 
 $end
